@@ -1,8 +1,10 @@
-#!/usr/bin/python
-
 """
 pyfsdb.Fsdb - class for reading and writing Fsdb files
 
+See pyfsdb.Fsdb for API documentation.
+
+Getting started hints:
+
 # creating a Fsdb object attached to a file to read:
 f = pyfsdb.Fsdb("data.fsdb")
 
@@ -47,6 +49,8 @@ results = f.filter(myfilt)
 """
 
 import sys
+import os
+import io
 
 if sys.version_info[0] < 3:
     raise Exception("Must be using Python 3")
@@ -68,12 +72,14 @@ incoming_type_converters = {
     # python doesn't have different float/double sizes
     "f": float,
     "d": float,
-    # we leave strings (c and C) alone
+    # we leave strings as strings
+    "a": str,
 }
 
 outgoing_type_converters = {
     int: "l",
     float: "d",
+    str: "a",
 }
 
 
@@ -100,12 +106,14 @@ class Fsdb(object):
     _out_header_line = None
     _out_column_names = []
     _out_column_names_set = False
-    _out_separator = "\t"
-    _out_separator_token = "t"
+    _out_separator = None
+    _out_separator_token = None
     _out_command_line = "____BROKEN____"  # ick, magic
     _save_command_history = True
     _handle_compressed = True
     _compression_checked = False
+    _commands = False
+    _seekable = True
 
     def __init__(
         self,
@@ -130,15 +138,19 @@ class Fsdb(object):
         RETURN_AS_DICTIONARY to return dictionary based rows with
         indexes as columns (this is slower).
 
-        If `pass_comments` is True and both an input and output file
+        If `pass_comments` is "y" and both an input and output file
         handle are available, any comments read in while reading
         are printed to the output.
 
+        If `save_command_history` is True, then comments will be saved
+        to the comments attribute.
+
         `converters` may be passed in as an array or dict of
         converters to call (such as int, float, etc)
 
         If `handle_compressed` is True (the default), the class will
         do its best to handle compressed formats: bz2, gzip, and xz (lzma).
+
         """
 
         self.return_type = return_type
@@ -172,6 +184,8 @@ class Fsdb(object):
             self.out_command_line = out_command_line
 
         self._comments = []
+        self.__real_next__ = None
+        self.unpacker = None
 
     @property
     def file_handle(self):
@@ -187,7 +201,7 @@ class Fsdb(object):
     @property
     def headers(self):
         "Headers for the file handle being read."
-        self.maybe_open_filehandle()
+        self.__maybe_open_filehandle()
         if not self._headers:
             self.read_header()
         return self._headers
@@ -199,7 +213,7 @@ class Fsdb(object):
     @property
     def header_line(self):
         "The top #fsdb header line in the file being read."
-        self.maybe_open_filehandle()
+        self.__maybe_open_filehandle()
         if not self._header_line:
             self.read_header()
         return self._header_line
@@ -211,14 +225,14 @@ class Fsdb(object):
     @property
     def column_names(self):
         "An array of column names for the file being read"
-        self.maybe_open_filehandle()
+        self.__maybe_open_filehandle()
         if not self._column_names:
             self.read_header()
         return list(self._column_names.keys())
 
     @column_names.setter
     def column_names(self, values):
-        mapping = self.__create_column_name_mapping__(values)
+        self.__create_column_name_mapping__(values)
         self._header_line = self.create_header_line(
             columns=values, separator_token=self._separator_token
         )
@@ -232,7 +246,7 @@ class Fsdb(object):
         be the 't' character and the separator would be '\t'.
 
         Changing this will also change the stored separator_token value."""
-        self.maybe_open_filehandle()
+        self.__maybe_open_filehandle()
         if not self._separator:
             self.read_header()
         return self._separator
@@ -254,7 +268,7 @@ class Fsdb(object):
         be the 't' character and the separator would be '\t'.
 
         Changing this will also change the stored separator value."""
-        self.maybe_open_filehandle()
+        self.__maybe_open_filehandle()
         if not self._separator_token:
             self.read_header()
         return self._separator_token
@@ -333,7 +347,7 @@ class Fsdb(object):
         elif self.out_file_handle:
             try:
                 self._out_file = self._out_file_handle.name
-            except:
+            except Exception:
                 pass
 
     @property
@@ -375,7 +389,7 @@ class Fsdb(object):
     @property
     def out_header_line(self):
         "The top #fsdb header line to write to the output."
-        self.maybe_open_filehandle()
+        self.__maybe_open_filehandle()
         self._out_header_line = self.create_header_line()
         return self._out_header_line
 
@@ -411,12 +425,42 @@ class Fsdb(object):
             self._out_column_names = self.column_names
         return self._out_column_names
 
+    @property
+    def comments(self):
+        """A list of comments seen in the document"""
+        return self._comments
+
+    @property
+    def commands(self):
+        "A list of commands that generated this file (if possible)"
+        return self.parse_commands()
+
     @out_column_names.setter
     def out_column_names(self, values):
-        mapping = self.__create_column_name_mapping__(values)
+        "The column names to be used in output FSDB content"
+        self.__create_column_name_mapping__(values)
+
+    @property
+    def converters(self):
+        """The list of conversion routines.
+
+        It may be an array, with a converter per column,
+        or a dict with a convert per named column.
+
+        This must be set before the file is opened/read.
+
+        Useful converted may include int, float, etc.
+
+        Note: if a converter throws an exception, a value of None will
+        be placed into the returned row instead.
+        """
+        return self._converters
+
+    @converters.setter
+    def converters(self, new_converters):
+        self._converters = new_converters
 
     # support functions
-
     def create_header_line(self, columns=None, separator_token=None, init_row=None):
         "Returns a header string for the stored column_names and separator/separator_token."
         if not columns:
@@ -468,26 +512,6 @@ class Fsdb(object):
 
         return header_line
 
-    @property
-    def converters(self):
-        """The list of conversion routines.
-
-        It may be an array, with a converter per column,
-        or a dict with a convert per named column.
-
-        This must be set before the file is opened/read.
-
-        Useful converted may include int, float, etc.
-
-        Note: if a converter throws an exception, a value of None will
-        be placed into the returned row instead.
-        """
-        return self._converters
-
-    @converters.setter
-    def converters(self, new_converters):
-        self._converters = new_converters
-
     def _convert_converters(self):
         if isinstance(self._converters, dict):
             converters = []
@@ -503,7 +527,7 @@ class Fsdb(object):
 
     def guess_converters(self, example_row):
         """Returns a best-guess effort list of converters after determining
-        if floats/ints exist in the dataset"""
+        if floats/ints exist in the example_row"""
         converters = {}
         for column in example_row:
             try:
@@ -517,16 +541,23 @@ class Fsdb(object):
                     pass
         return converters
 
+    def init_output_from(self, other_fsdb):
+        "Copies columns from an input FSDB object to an output object's configuration"
+        self.out_column_names = other_fsdb.column_names
+        self.out_separator = other_fsdb.separator
+        self.converters = other_fsdb.converters
+        return self
+
     # column accessor helpers
     def get_column_number(self, column_name):
         "Given a column_name, returns its integer index into an array of values."
-        self.maybe_open_filehandle()
+        self.__maybe_open_filehandle()
         self.read_header()
         return self._column_names[column_name]
 
     def get_column_numbers(self, column_names):
         "Given a list of column_names, returns a list of integer index into an array of values."
-        self.maybe_open_filehandle()
+        self.__maybe_open_filehandle()
         self.read_header()
         column_numbers = []
         for name in column_names:
@@ -535,27 +566,36 @@ class Fsdb(object):
 
     def get_column_name(self, column_number):
         "Given an integer column number, returns its column name."
-        self.maybe_open_filehandle()
+        self.__maybe_open_filehandle()
         self.read_header()
         return self.column_nums[column_number]
 
-    def set_iterator_function(self):
-        "XXX: change this to an property"
+    def set_iterator_function(self, return_type=None):
+        """Changes the internal iterator function based on the specified return type.
+
+        Note that this is unlikely safe to use on the fly"""
+
+        if return_type:
+            self.return_type = return_type
+
         if self.return_type == RETURN_AS_DICTIONARY:
-            self.__next__ = self._next_as_dict
+            self.__real_next__ = self._next_as_dict
         else:
-            self.__next__ = self._next_as_array
+            self.__real_next__ = self._next_as_array
+
+        # sigh this doesn't work as the python iterator code caches the next pointer
+        # TODO: see if we can override this
+        self.__next__ = self.__real_next__
 
     def __iter__(self):
         """Returns an iterator object for looping over an fsdb file."""
+        self.bootstrap()
         if not self.filename and not self.file_handle:
             raise ValueError("No filename or handle currently available for reading")
         # XXX: throw error on -1 parse
         return self
 
-    def maybe_open_filehandle(self, mode="r"):
-        "Internal"
-
+    def __maybe_open_filehandle(self):
         # the simple case:
         if self.file_handle and (
             self._compression_checked or not self._handle_compressed
@@ -567,7 +607,7 @@ class Fsdb(object):
 
         # the simple case, open it if we don't need to detect compressed
         if not self.file_handle and self.filename and not self._handle_compressed:
-            self.file_handle = open(self.filename, mode)
+            self.file_handle = open(self.filename, "rb")
             return self.file_handle
 
         # wrap this in case anything at all fails:
@@ -600,17 +640,22 @@ class Fsdb(object):
                                 if filetype == "gz":
                                     import gzip
 
-                                    self.file_handle = gzip.open(filename, "rt")
+                                    self._seekable = False  # unlikely
+                                    self.file_handle = gzip.open(filename, "rb")
                                     return self.file_handle
                                 elif filetype == "bz2":
                                     import bz2
 
-                                    self.file_handle = bz2.open(filename, "rt")
+                                    self._seekable = False  # unlikely
+                                    self.file_handle = bz2.open(filename, "rb")
                                     return self.file_handle
                                 elif filetype == "xz":
                                     import lzma
 
-                                    self.file_handle = lzma.open(filename, "rt")
+                                    self._seekable = (
+                                        False  # say they are when they're not
+                                    )
+                                    self.file_handle = lzma.open(filename, "rb")
                                     return self.file_handle
                             except Exception:
                                 sys.stderr.write(
@@ -622,30 +667,35 @@ class Fsdb(object):
 
         # fall back to just opening it and hope its raw
         if self.filename:
-            self.file_handle = open(self.filename, mode)
+            self.file_handle = open(self.filename, "rb")
 
         return self.file_handle
 
+    def bootstrap(self):
+        "Performs initialization and sets up function handlers"
+        fh = self.__maybe_open_filehandle()
+
+        if not fh:
+            raise ValueError("no filehandle specified")
+
+        if not self._header_line:
+            self.read_header()
+
+        self._convert_converters()
+
+        self.set_iterator_function()
+
     def __next__(self):
         """Returns the next array of data from an fsdb file.
         Returns an array by default, or a dictionary if return_type
         was set to pyfsdb.RETURN_AS_DICTIONARY."""
 
-        fh = self.maybe_open_filehandle()
-        if not self._header_line:
-            self.read_header()
+        # if we've initialized already, use the real path:
+        if self.__real_next__:
+            return self.__real_next__()
 
-        self._convert_converters()
-
-        if self.return_type == RETURN_AS_DICTIONARY:
-            self.__next__ = self._next_as_dict
-        else:
-            self.__next__ = self._next_as_array
-
-        if not fh:
-            return None
-
-        return self.__next__()
+        self.bootstrap()
+        return self.__real_next__()
 
     def _handle_comment(self, line):
         """Handle a comment by printing it, possibly with header init first,
@@ -662,7 +712,10 @@ class Fsdb(object):
                 self._out_file_handle.write(line)
             else:
                 self._comments.append(line)
-        return next(self.fileh)
+        elif self._pass_comments == "y":
+            self._comments.append(line)
+
+        return self._next_line()
 
     def _convert_array_values(self, row):
         for (n, converter) in enumerate(self._converters):
@@ -675,21 +728,48 @@ class Fsdb(object):
                     row[n] = None
         return row
 
+    def _next_line(self):
+        # return next(self.fileh)
+        return self._next_line_binary()
+
+    def _next_line_binary(self):
+        line = next(self.fileh)
+        if isinstance(line, bytes):
+            line = line.decode()
+
+        while line and line[0] == "#":
+            line = self._handle_comment(line)
+            if isinstance(line, bytes):
+                line = line.decode()
+
+        return line
+
     def _next_as_array(self):
         """Return the next object as an array of columns."""
 
-        line = next(self.fileh)
-        while line and line[0] == "#":
-            line = self._handle_comment(line)
+        if self.separator == "m":
+            # msgpack encoded
+            import msgpack
+
+            if not self.unpacker:
+                self.unpacker = msgpack.Unpacker(
+                    getattr(self.fileh, "buffer", self.fileh)
+                )
+            self._current_row = next(self.unpacker)
+
+        else:
+            # normal text file
+            line = self._next_line()
+
+            # return an array of data
+            self.current_line = line
+            self._current_row = line.rstrip("\n\r").split(self.separator)
+            if len(self._current_row) < len(self._column_names):
+                n = (len(self._column_names)) - len(self._current_row)
+                self._current_row.extend([""] * n)
+            if self._converters:
+                self._current_row = self._convert_array_values(self._current_row)
 
-        # return an array of data
-        self.current_line = line
-        self._current_row = line.rstrip("\n\r").split(self.separator)
-        if len(self._current_row) < len(self._column_names):
-            n = (len(self._column_names)) - len(self._current_row)
-            self._current_row.extend([""] * n)
-        if self._converters:
-            self._current_row = self._convert_array_values(self._current_row)
         return self._current_row
 
     def _next_as_dict(self):
@@ -714,10 +794,10 @@ class Fsdb(object):
         Using a generator is faster than using the Fsdb object
         as a iterator."""
 
-        fh = self.maybe_open_filehandle()
+        self.__maybe_open_filehandle()
 
         try:
-            line = next(self.fileh)
+            line = self._next_line()
             while line:
                 while line and line[0] == "#":
                     line = self._handle_comment(line)
@@ -726,8 +806,8 @@ class Fsdb(object):
                 self.current_line = line
                 self._current_row = line.rstrip("\n\r").split(self.separator)
                 yield self._current_row
-                line = next(self.fileh)
-        except StopIteration as e:
+                line = self._next_line()
+        except StopIteration:
             return
 
     def next_as_dict(self):
@@ -736,10 +816,10 @@ class Fsdb(object):
         Using a generator is faster than using the Fsdb object
         as a iterator."""
 
-        fh = self.maybe_open_filehandle()
+        self.__maybe_open_filehandle()
 
         try:
-            line = next(self.fileh)
+            line = self._next_line()
             while line:
                 while line and line[0] == "#":
                     line = self._handle_comment(line)
@@ -753,8 +833,8 @@ class Fsdb(object):
                     return_dict[self.column_nums[index]] = self._current_row[index]
 
                 yield return_dict
-                line = next(self.fileh)
-        except StopIteration as e:
+                line = self._next_line()
+        except StopIteration:
             return
 
     def parse_separator(self, separator=None):
@@ -765,6 +845,8 @@ class Fsdb(object):
             return "  "
         elif separator == "s":
             return " "
+        elif separator == "m":
+            return "m"
         elif separator[0] == "c":
             return separator[1:]
         elif separator[0] == "C":
@@ -773,6 +855,7 @@ class Fsdb(object):
             return chr(int("0x" + separator[1:], 0))
         elif separator[0] == "X":  # won't handle multiples like manual says
             return chr(int("0x" + separator[1:], 0))
+
         elif separator == "D":
             # python NONE to splits on all white space
             return None
@@ -788,9 +871,11 @@ class Fsdb(object):
             return "S"
         elif separator_token == " ":
             return "s"
+        elif separator_token == "m":
+            return "m"
         elif len(separator_token) == 1:
             return "C" + separator_token
-        elif separator_token == None:
+        elif separator_token is None:
             # XXX
             separator_token = "D"
 
@@ -819,8 +904,38 @@ class Fsdb(object):
             return [0, self._mapping]
         self._have_read_header = True
 
+        # we read a byte at a time to allow binary beyond the header
+        # ie, next() doesn't work on mostly binary files
+        # self.fileh.buffer is the raw (binary mode) buffer of normal files
+        readh = getattr(self.fileh, "buffer", self.fileh)
+
+        # how to read multiple utf-8 bytes as needed taken from
+        # https://stackoverflow.com/questions/15199675/reading-utf-8-strings-from-a-binary-file
+        _lead_byte_to_count = []
+        for i in range(256):
+            _lead_byte_to_count.append(
+                1 + (i >= 0xE0) + (i >= 0xF0) if 0xBF < i < 0xF5 else 0
+            )
+
         if not line:
-            line = next(self.file_handle)
+            line = ""
+            addition = None
+            while addition != "\n" and addition != "":
+                addition = readh.read(1)
+                if len(addition) != 1:
+                    break
+                read_count = _lead_byte_to_count[ord(addition)]
+                if read_count:
+                    addition += readh.read(read_count)
+                if getattr(addition, "decode", False):
+                    try:
+                        addition = addition.decode()
+                    except UnicodeDecodeError:
+                        print("failed utf-8")
+                line += addition
+
+            # place = self.fileh.tell()
+
         self._header_line = line
         self._headers = [self._header_line]
 
@@ -830,7 +945,7 @@ class Fsdb(object):
 
         # should we use argparse here?
         argn = 1
-        separator = None  # (D)efault is all white space
+        self._separator = None  # (D)efault is all white space
         while args[argn][0] == "-":
             if args[argn] == "-F":
                 argn += 1
@@ -841,6 +956,9 @@ class Fsdb(object):
             argn += 1
 
         self._separator = self.parse_separator(self._separator_token)
+        if not self._out_separator:
+            self._out_separator_token = self._separator_token
+            self._out_separator = self._separator
 
         # join the remainder of the arguments back together to split
         # by the correct separator
@@ -900,7 +1018,7 @@ class Fsdb(object):
             for column in column_names:
                 try:
                     df[column] = pandas.to_numeric(df[column])
-                except:
+                except Exception:
                     pass
 
             return df
@@ -916,12 +1034,16 @@ class Fsdb(object):
 
     def put_pandas(self, df):
         "saves a pandas dataframe to the output file"
-        import pandas
-
         if not self._out_column_names:
             self.out_column_names = df.columns.values.tolist()
         self._write_header_line()
-        df.to_csv(self._out_file_handle, sep="\t", header=False, index=False)
+        df.to_csv(
+            self._out_file_handle,
+            sep="\t",
+            header=False,
+            index=False,
+            columns=self.out_column_names,
+        )
 
     def comment(self, line):
         """Add a comment to an ouput FSDB file
@@ -977,18 +1099,41 @@ class Fsdb(object):
 
     def _write_header_line(self, init_row=None):
         if not self._out_column_names and not self._header_line:
+            raise ValueError("no output column names specified")
             return  # we're unable to at this point
 
+        if not self._out_separator:
+            self._out_separator = "\t"  # default to tab
+            self._out_separator_type = "t"  # default to tab
+
         # maybe construct it
         if not self._out_header_line and self._out_column_names:
             self._out_header_line = self.create_header_line(init_row=init_row)
 
-        # write out the correct header
+        # start by assuming copy the original
+        output_header = self._header_line
         if self._out_header_line:
-            self._out_file_handle.write(self._out_header_line)
-        elif self._header_line:
-            # assuming copy the original
-            self._out_file_handle.write(self._header_line)
+            # otherwise use a specified one
+            output_header = self._out_header_line
+
+        if self.out_separator == "m":
+            # switch to the internal binary buffer if possible
+            self.out_file_handle = getattr(
+                self.out_file_handle, "buffer", self.out_file_handle
+            )
+            self._out_file_handle.write(bytes(output_header, encoding="utf-8"))
+        else:
+            self._out_file_handle.write(output_header)
+
+        # if we're in msgpack/binary mode, stop here
+        if self.out_separator == "m":
+            self.append = self._append_msgpack
+            self._header_written = True
+            # TODO: ie, we need to deal with storing comments at some point
+            self._pass_comments = False
+            # TODO: and command history
+            self._save_command_history = False
+            return
 
         # see if we have any early stored comments
         if self._pass_comments == "y" and self._comments:
@@ -1015,23 +1160,136 @@ class Fsdb(object):
             row = [row[x] for x in self.out_column_names]
         if self._write_nones_as_blanks:
             for i in range(0, len(row)):
-                if row[i] == None:
+                if row[i] is None:
                     row[i] = ""
         self._out_file_handle.write(self._out_separator.join(map(str, row)) + "\n")
 
-    # backwards compatible ... don't use
-    def write_row(self, row=None):
-        self.append(row)
+    def _append_msgpack(self, row=None):
+        import msgpack
 
-    def write_finish(self):
-        self.close()
+        self._out_file_handle.write(msgpack.packb(row, use_bin_type=True))
 
-    def close(self):
+    def import_comments(self, from_fsdb):
+        "Add comments from an existing FSDB to this one"
+        for comment in from_fsdb._comments:
+            self._comments.append(comment)
+
+    def get_file_size(self):
+        "Returns the size of the input file, if possible"
+        return os.stat(self.fileh.name).st_size
+
+    def read_commands_ahead(self):
+        """reads the command list at the bottom of the input stream if the input stream can seek.
+
+        returns a list of commands found in comments in the input stream
+        returns None when the input is not seekable"""
+
+        self.__maybe_open_filehandle()
+
+        if not self._seekable or not self.file_handle.seekable():
+            return None
+
+        guess_length = 512
+
+        # save our spot
+        position = self.file_handle.seek(0)
+
+        # get the file size
+        try:
+            file_size = self.get_file_size()
+        except Exception:
+            return None
+        multiplier = 1
+
+        # move to the bottom minus the guess length
+        start_seek = file_size - guess_length
+        if start_seek < file_size:
+            guess_length = file_size
+        self.file_handle.seek(file_size - guess_length)
+
+        # poor man's search from the back
+        while True:
+            data = self.file_handle.read(guess_length * multiplier)
+            if isinstance(data, bytes):
+                data = data.decode()
+            first_newline = data.find("\n")
+
+            if first_newline != -1 and (
+                first_newline == len(data) - 1 or data[first_newline + 1] == "#"
+            ):
+                # keep rewinding (x2 since we read x1 in already)
+                multiplier += 1
+
+                if file_size - guess_length * multiplier <= 0:
+                    return None
+
+                self.file_handle.seek(file_size - guess_length * multiplier)
+                continue
+
+            break
+
+        # now extract all the commands from where we've found things
+        commands = []
+        iowrapper = io.StringIO(data)
+
+        import re
+
+        command_matcher = re.compile("# +\\| (.*)")
+
+        for line in iowrapper:
+            result = command_matcher.match(line)
+            if result:
+                commands.append(result.group(1))
+
+        self.file_handle.seek(position)
+        return commands
+
+    def parse_commands(self):
+        """parses the list of stored comments for any saved commands
+
+        Note: Assumes saved commands will be prefixed with '#  |' per convention
+
+        Returns a list of strings when commands can be found.
+
+        Returns None when we don't have the information yet, such as when
+        we have a non-seekable stream input.
+        """
+
+        self.__maybe_open_filehandle()
+
+        # we already have some stored
+        if self._commands:
+            return self._commands
+
+        # see if we can read them from the end of the file
+        if not self._comments:
+            if self._seekable and self.file_handle.seekable():
+                self._commands = self.read_commands_ahead()
+                return self._commands
+            return None
+
+        import re
+
+        command_matcher = re.compile("# +\\| (.*)")
+
+        # parse them from the comments
+        self._commands = []
+        for comment in self._comments:
+            results = command_matcher.match(comment)
+            if results:
+                self._commands.append(results.group(1))
+
+        return self._commands
+
+    def close(self, copy_comments_from=None):
         """Writes final processing command comment to the output file and closes it."""
         if self.fileh:
             self.fileh.close()
             self.fileh = None
 
+        if copy_comments_from:
+            self.import_comments(copy_comments_from)
+
         if self._out_file_handle:
             # ignore closing errors
             try:
@@ -1040,9 +1298,9 @@ class Fsdb(object):
                 if self._save_command_history and self.out_command_line:
                     for comment_line in self._comments:
                         self._out_file_handle.write(comment_line)
-                    self._out_file_handle.write("#   | " + self.out_command_line + "\n")
+                    self._out_file_handle.write("#  | " + self.out_command_line + "\n")
                 self._out_file_handle.close()
-            except:
+            except Exception:
                 pass
             self._out_file_handle = None
 
