@@ -1,5 +1,3 @@
-#!/usr/bin/python3
-
 """fsdb2many converts a single FSDB file into many, by creating
 other file names based on a column of the original."""
 
@@ -58,7 +56,7 @@ def main():
             # new value, so open a new file handle to save data for it
             file_name = re.sub("[^-.0-9a-zA-Z_]", "_", str(value))
             outh = pyfsdb.Fsdb(out_file=(args.output_pattern % file_name))
-            outh.column_names = inh.column_names
+            outh.init_output_from(inh)
             out_handles[value] = outh
 
         # save the row to the file based on its value
