@@ -1,5 +1,3 @@
-#!/usr/bin/python3
-
 """This script expects to augment a FSDB stream (or file) with
    additional information found in another FSDB file.  The second file
    is loaded entirely into memory in order to accomplish this.  Any
@@ -7,7 +5,7 @@
    file will be augmented to using data from the 'values' columns is
    the augment file.
 
-   This duplicates dbjoin to a large extent, but dbaugment is faster
+   This duplicates dbjoin to a large extent, but pdbaugment is faster
    when one side is small because it avoids sorting.  dbaugment can
    also operate on streaming data, since sorting isn't required.
 """
@@ -179,7 +177,9 @@ def main():
         stash_row(cache, args.keys, row)
 
     # read in stream file, and augment each row with the new columns
-    streamh = pyfsdb.Fsdb(file_handle=args.stream_file)
+    streamh = pyfsdb.Fsdb(
+        file_handle=args.stream_file, out_file_handle=args.output_file
+    )
 
     # determine which columns need to be added on (potentially all)
     if args.include_all_values or (
@@ -194,11 +194,10 @@ def main():
         args.values = []
 
     # create the output stream to store the data
-    outh = pyfsdb.Fsdb(out_file_handle=args.output_file)
     other_columns = []
     if args.mark_new:
         other_columns = [args.mark_new]
-    outh.out_column_names = streamh.column_names + args.values + other_columns
+    streamh.out_column_names = streamh.column_names + args.values + other_columns
 
     key_columns = streamh.get_column_numbers(args.keys)
 
@@ -217,12 +216,12 @@ def main():
         elif args.mark_new:
             row.extend([None] * len(args.values) + [1])
 
-        outh.append(row)
+        streamh.append(row)
 
     # Now loop through all data adding any rows
     if args.include_unmatched_augment_rows:
         dump_remaining(
-            outh,
+            streamh,
             cache,
             # total original column numbers - key count
             len(streamh.column_names) - len(args.keys),
