@@ -1,5 +1,3 @@
-#!/usr/bin/python3
-
 """Outputs a python-string formatted line for every input FSDB row,
 with column names acting as variables into the format string."""
 
@@ -55,6 +53,10 @@ def main():
     format_string = args.format
 
     for row in inh:
+        # convert Nones
+        for column in row:
+            if row[column] is None:
+                row[column] = ""
         outh.write(format_string.format(**row) + "\n")
 
 
